@@ -1,5 +1,4 @@
 #include <stdint.h>
-#include <limits.h>
 #include <fs/fat32.h>
 #include <lib/misc.h>
 #include <drivers/disk.h>
@@ -292,14 +291,24 @@ static int read_cluster_from_map(struct fat32_context *context, uint32_t cluster
             }
             break;
         }
-        case 16:
+        case 16: {
             *out = 0;
-            volume_read(context->part, out, fat_base + (uint64_t)cluster * sizeof(uint16_t), sizeof(uint16_t));
+            uint64_t offset = (uint64_t)cluster * sizeof(uint16_t);
+            if (offset + sizeof(uint16_t) > fat_size) {
+                return -1;
+            }
+            volume_read(context->part, out, fat_base + offset, sizeof(uint16_t));
             break;
-        case 32:
-            volume_read(context->part, out, fat_base + (uint64_t)cluster * sizeof(uint32_t), sizeof(uint32_t));
+        }
+        case 32: {
+            uint64_t offset = (uint64_t)cluster * sizeof(uint32_t);
+            if (offset + sizeof(uint32_t) > fat_size) {
+                return -1;
+            }
+            volume_read(context->part, out, fat_base + offset, sizeof(uint32_t));
             *out &= 0x0fffffff;
             break;
+        }
         default:
             __builtin_unreachable();
     }
@@ -654,10 +663,10 @@ struct file_handle *fat32_open(struct volume *part, const char *path) {
 
         for (unsigned int i = 0; i < SIZEOF_ARRAY(current_part) - 1; i++) {
             // Check for overflow before computing path index
-            if (current_index > UINT_MAX - i) {
+            unsigned int path_idx;
+            if (__builtin_add_overflow(i, current_index, &path_idx)) {
                 return NULL;  // Path index would overflow
             }
-            unsigned int path_idx = i + current_index;
 
             if (path[path_idx] == 0) {
                 memcpy(current_part, path + current_index, i);
@@ -671,10 +680,11 @@ struct file_handle *fat32_open(struct volume *part, const char *path) {
                 memcpy(current_part, path + current_index, i);
                 current_part[i] = 0;
                 // Check for overflow before updating current_index
-                if (current_index > UINT_MAX - i - 1) {
+                unsigned int new_index;
+                if (__builtin_add_overflow(current_index, i + 1, &new_index)) {
                     return NULL;  // current_index would overflow
                 }
-                current_index += i + 1;
+                current_index = new_index;
                 expect_directory = true;
                 found_terminator = true;
                 break;
